"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BchatPasswordPrompt = exports.MAX_LOGIN_TRIES = void 0;
const react_1 = __importDefault(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const icon_1 = require("./icon");
const styled_components_1 = require("styled-components");
const auto_bind_1 = __importDefault(require("auto-bind"));
const BchatButton_1 = require("./basic/BchatButton");
const BchatSpinner_1 = require("./basic/BchatSpinner");
exports.MAX_LOGIN_TRIES = 3;
class BchatPasswordPromptInner extends react_1.default.PureComponent {
    inputRef;
    constructor(props) {
        super(props);
        this.state = {
            error: '',
            errorCount: 0,
            clearDataView: false,
            loading: false,
        };
        (0, auto_bind_1.default)(this);
    }
    componentDidMount() {
        setTimeout(() => {
            this.inputRef?.focus();
        }, 100);
    }
    render() {
        const showResetElements = this.state.errorCount >= exports.MAX_LOGIN_TRIES;
        let checkError = this.state.error;
        if (checkError) {
        }
        const wrapperClass = this.state.clearDataView
            ? 'clear-data-wrapper'
            : 'password-prompt-wrapper';
        const containerClass = this.state.clearDataView
            ? 'clear-data-container'
            : 'password-prompt-container';
        const infoAreaClass = this.state.clearDataView ? 'warning-info-area' : 'password-info-area';
        const infoTitle = this.state.clearDataView
            ? window.i18n('clearAllData')
            : window.i18n('passwordViewTitle');
        const buttonGroup = this.state.clearDataView
            ? this.renderClearDataViewButtons()
            : this.renderPasswordViewButtons();
        const featureElement = this.state.clearDataView ? (react_1.default.createElement("p", { className: "text-center" }, window.i18n('deleteAccountWarning'))) : (react_1.default.createElement("input", { id: "password-prompt-input", type: "password", defaultValue: "", placeholder: ' ', minLength: 4, maxLength: 26, onKeyUp: this.onKeyUp, ref: input => {
                this.inputRef = input;
            } }));
        const infoIcon = this.state.clearDataView ? (react_1.default.createElement(icon_1.BchatIcon, { iconType: "warning", iconSize: 35, iconColor: "#ce0000" })) : (react_1.default.createElement(icon_1.BchatIcon, { iconType: "lock", iconSize: 30 }));
        const errorSection = !this.state.clearDataView && (react_1.default.createElement("div", { className: "password-prompt-error-section" }, this.state.error && (react_1.default.createElement(react_1.default.Fragment, null, showResetElements ? (react_1.default.createElement("div", { className: "bchat-label warningBg" }, window.i18n('maxPasswordAttempts'))) : (react_1.default.createElement("div", { className: "bchat-label primary" }, this.state.error))))));
        const spinner = this.state.loading ? react_1.default.createElement(BchatSpinner_1.BchatSpinner, { loading: true }) : null;
        return (react_1.default.createElement("div", { className: "password" },
            react_1.default.createElement("div", { className: wrapperClass },
                react_1.default.createElement("div", { className: containerClass },
                    react_1.default.createElement("div", { className: infoAreaClass },
                        infoIcon,
                        react_1.default.createElement("h1", null, infoTitle)),
                    spinner || featureElement,
                    errorSection,
                    buttonGroup))));
    }
    onKeyUp(event) {
        switch (event.key) {
            case 'Enter':
                this.initLogin();
                break;
            default:
        }
        event.preventDefault();
    }
    async onLogin(passPhrase) {
        const passPhraseTrimmed = passPhrase.trim();
        try {
            await window.onLogin(passPhraseTrimmed);
        }
        catch (error) {
            this.setState({
                errorCount: this.state.errorCount + 1,
            });
            this.setState({ error });
            global.setTimeout(() => {
                document.getElementById('password-prompt-input')?.focus();
            }, 50);
        }
        this.setState({
            loading: false,
        });
    }
    initLogin() {
        this.setState({
            loading: true,
        });
        const passPhrase = String(this.inputRef.value);
        global.setTimeout(() => this.onLogin(passPhrase), 100);
    }
    initClearDataView() {
        this.setState({
            error: '',
            errorCount: 0,
            clearDataView: true,
        });
    }
    renderPasswordViewButtons() {
        const showResetElements = this.state.errorCount >= exports.MAX_LOGIN_TRIES;
        return (react_1.default.createElement("div", { className: (0, classnames_1.default)(showResetElements && 'button-group') },
            showResetElements && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('resetDatabase'), buttonType: BchatButton_1.BchatButtonType.BrandOutline, buttonColor: BchatButton_1.BchatButtonColor.White, onClick: this.initClearDataView }))),
            react_1.default.createElement("div", { className: "unlock-button" },
                react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('unlock'), buttonType: BchatButton_1.BchatButtonType.BrandOutline, buttonColor: BchatButton_1.BchatButtonColor.Green, onClick: this.initLogin }))));
    }
    renderClearDataViewButtons() {
        return (react_1.default.createElement("div", { className: "button-group" },
            react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('cancel'), buttonType: BchatButton_1.BchatButtonType.Default, buttonColor: BchatButton_1.BchatButtonColor.Primary, onClick: () => {
                    this.setState({ clearDataView: false });
                } }),
            react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('clearAllData'), buttonType: BchatButton_1.BchatButtonType.Default, buttonColor: BchatButton_1.BchatButtonColor.Danger, onClick: window.clearLocalData })));
    }
}
exports.BchatPasswordPrompt = (0, styled_components_1.withTheme)(BchatPasswordPromptInner);
