"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AvatarPlaceHolder = void 0;
const react_1 = __importStar(require("react"));
const getInitials_1 = require("../../../util/getInitials");
const sha512FromPubkey = async (pubkey) => {
    const buf = await crypto.subtle.digest('SHA-512', new TextEncoder().encode(pubkey));
    return Array.prototype.map
        .call(new Uint8Array(buf), (x) => ('00' + x.toString(16)).slice(-2))
        .join('');
};
const cachedHashes = new Map();
const avatarPlaceholderColors = ['#FF5722', '#2979FB', '#FF6663', '#009688', "#FE64A3", '#00B1FF', '#673AB7', '#E91E63', '#9C27B0'];
const avatarBorderColor = '#00000059';
function useHashBasedOnPubkey(pubkey) {
    const [hash, setHash] = (0, react_1.useState)(undefined);
    const [loading, setIsLoading] = (0, react_1.useState)(true);
    (0, react_1.useEffect)(() => {
        const cachedHash = cachedHashes.get(pubkey);
        if (cachedHash) {
            setHash(cachedHash);
            setIsLoading(false);
            return;
        }
        setIsLoading(true);
        let isInProgress = true;
        if (!pubkey) {
            if (isInProgress) {
                setIsLoading(false);
                setHash(undefined);
            }
            return;
        }
        void sha512FromPubkey(pubkey).then(sha => {
            if (isInProgress) {
                setIsLoading(false);
                if (sha) {
                    const hashed = parseInt(sha.substring(0, 12), 16) || 0;
                    setHash(hashed);
                    cachedHashes.set(pubkey, hashed);
                    return;
                }
                setHash(undefined);
            }
        });
        return () => {
            isInProgress = false;
        };
    }, [pubkey]);
    return { loading, hash };
}
const AvatarPlaceHolder = (props) => {
    const { pubkey, diameter, name } = props;
    const { hash, loading } = useHashBasedOnPubkey(pubkey);
    const viewBox = `0 0 ${diameter} ${diameter}`;
    if (loading || !hash) {
        return (react_1.default.createElement("svg", { viewBox: viewBox },
            react_1.default.createElement("g", { id: "UrTavla" },
                react_1.default.createElement("rect", { rx: 10, ry: 10, fill: "#d2d2d3", width: diameter, height: diameter, stroke: avatarBorderColor, strokeWidth: "1" }))));
    }
    const initials = (0, getInitials_1.getInitials)(name);
    const fontSize = Math.floor(initials.length > 1 ? diameter * 0.4 : diameter * 0.5);
    const bgColorIndex = hash % avatarPlaceholderColors.length;
    const bgColor = avatarPlaceholderColors[bgColorIndex];
    return (react_1.default.createElement("svg", { viewBox: viewBox },
        react_1.default.createElement("g", { id: "UrTavla" },
            react_1.default.createElement("rect", { rx: 10, ry: 10, width: diameter, height: diameter, fill: bgColor, stroke: avatarBorderColor, strokeWidth: "1" }),
            react_1.default.createElement("text", { fontSize: fontSize, x: "50%", y: "50%", fill: "white", textAnchor: "middle", stroke: "white", strokeWidth: 1, alignmentBaseline: "central", height: fontSize }, initials))));
};
exports.AvatarPlaceHolder = AvatarPlaceHolder;
