"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InConversationCallContainer = exports.VideoLoadingSpinner = void 0;
const react_redux_1 = require("react-redux");
const react_1 = __importStar(require("react"));
const styled_components_1 = __importDefault(require("styled-components"));
const utils_1 = require("../../bchat/utils");
const call_1 = require("../../state/selectors/call");
const DraggableCallContainer_1 = require("./DraggableCallContainer");
const Avatar_1 = require("../avatar/Avatar");
const useVideoEventListener_1 = require("../../hooks/useVideoEventListener");
const useModuloWithTripleDots_1 = require("../../hooks/useModuloWithTripleDots");
const CallButtons_1 = require("./CallButtons");
const CallManager_1 = require("../../bchat/utils/calling/CallManager");
const useInterval_1 = __importDefault(require("react-use/lib/useInterval"));
const moment_1 = __importDefault(require("moment"));
const BchatSpinner_1 = require("../basic/BchatSpinner");
const VideoContainer = styled_components_1.default.div `
  height: 100%;
  width: 50%;
  z-index: 0;
  padding-top: 30px; // leave some space at the top for the connecting/duration of the current call
`;
const InConvoCallWindow = styled_components_1.default.div `
  padding: 1rem;
  display: flex;

  background-color: var(--color-inbox-background);

  flex-shrink: 1;
  min-height: 80px;
  align-items: center;
  flex-grow: 1;
`;
const RelativeCallWindow = styled_components_1.default.div `
  position: relative;
  height: 100%;
  display: flex;
  flex-grow: 1;
`;
const CenteredAvatarInConversation = styled_components_1.default.div `
  top: -50%;
  transform: translateY(-50%);
  position: relative;
  bottom: 0;
  left: 0;
  right: 50%;

  display: flex;
  justify-content: center;
  align-items: center;
`;
const StyledCenteredLabel = styled_components_1.default.div `
  position: absolute;
  left: 50%;
  transform: translateX(-50%);
  height: min-content;
  white-space: nowrap;
  color: var(--color-text);
  text-shadow: 0px 0px 8px white;
  z-index: 5;
`;
const RingingLabel = () => {
    const ongoingCallWithFocusedIsRinging = (0, react_redux_1.useSelector)(call_1.getCallWithFocusedConvoIsOffering);
    const modulatedStr = (0, useModuloWithTripleDots_1.useModuloWithTripleDots)(window.i18n('ringing'), 3, 1000);
    if (!ongoingCallWithFocusedIsRinging) {
        return null;
    }
    return react_1.default.createElement(StyledCenteredLabel, null, modulatedStr);
};
const ConnectingLabel = () => {
    const ongoingCallWithFocusedIsConnecting = (0, react_redux_1.useSelector)(call_1.getCallWithFocusedConvosIsConnecting);
    const modulatedStr = (0, useModuloWithTripleDots_1.useModuloWithTripleDots)(window.i18n('establishingConnection'), 3, 1000);
    if (!ongoingCallWithFocusedIsConnecting) {
        return null;
    }
    return react_1.default.createElement(StyledCenteredLabel, null, modulatedStr);
};
const DurationLabel = () => {
    const [callDuration, setCallDuration] = (0, react_1.useState)(undefined);
    const ongoingCallWithFocusedIsConnected = (0, react_redux_1.useSelector)(call_1.getCallWithFocusedConvosIsConnected);
    (0, useInterval_1.default)(() => {
        const duration = utils_1.CallManager.getCurrentCallDuration();
        if (duration) {
            setCallDuration(duration);
        }
    }, 100);
    if (!ongoingCallWithFocusedIsConnected || !callDuration || callDuration < 0) {
        return null;
    }
    const ms = callDuration * 1000;
    const d = moment_1.default.duration(ms);
    const dateString = Math.floor(d.asHours()) + moment_1.default.utc(ms).format(':mm:ss');
    return react_1.default.createElement(StyledCenteredLabel, null, dateString);
};
const StyledSpinner = styled_components_1.default.div `
  height: 100%;
  width: ${props => (props.fullWidth ? '100%' : '50%')};
  display: flex;
  justify-content: center;
  align-items: center;
  position: absolute;
  z-index: -1;
`;
const VideoLoadingSpinner = (props) => {
    return (react_1.default.createElement(StyledSpinner, { fullWidth: props.fullWidth },
        react_1.default.createElement(BchatSpinner_1.BchatSpinner, { loading: true })));
};
exports.VideoLoadingSpinner = VideoLoadingSpinner;
const InConversationCallContainer = () => {
    const isInFullScreen = (0, react_redux_1.useSelector)(call_1.getCallIsInFullScreen);
    const ongoingCallPubkey = (0, react_redux_1.useSelector)(call_1.getHasOngoingCallWithPubkey);
    const ongoingCallWithFocused = (0, react_redux_1.useSelector)(call_1.getHasOngoingCallWithFocusedConvo);
    const videoRefRemote = (0, react_1.useRef)(null);
    const videoRefLocal = (0, react_1.useRef)(null);
    const ourPubkey = utils_1.UserUtils.getOurPubKeyStrFromCache();
    const { currentConnectedAudioInputs, currentConnectedCameras, currentConnectedAudioOutputs, currentSelectedAudioOutput, localStream, localStreamVideoIsMuted, remoteStream, remoteStreamVideoIsMuted, isAudioMuted, isAudioOutputMuted, } = (0, useVideoEventListener_1.useVideoCallEventsListener)('InConversationCallContainer', true);
    if (videoRefRemote?.current && videoRefLocal?.current) {
        if (videoRefRemote.current.srcObject !== remoteStream) {
            videoRefRemote.current.srcObject = remoteStream;
        }
        if (videoRefLocal.current.srcObject !== localStream) {
            videoRefLocal.current.srcObject = localStream;
        }
        if (videoRefRemote.current) {
            if (currentSelectedAudioOutput === CallManager_1.DEVICE_DISABLED_DEVICE_ID) {
                videoRefRemote.current.muted = true;
            }
            else {
                void videoRefRemote.current?.setSinkId(currentSelectedAudioOutput);
                videoRefRemote.current.muted = false;
            }
        }
    }
    if (isInFullScreen && videoRefRemote.current) {
        videoRefRemote.current.muted = true;
    }
    if (!ongoingCallWithFocused || !ongoingCallPubkey) {
        return null;
    }
    return (react_1.default.createElement(InConvoCallWindow, null,
        react_1.default.createElement(RelativeCallWindow, null,
            react_1.default.createElement(RingingLabel, null),
            react_1.default.createElement(ConnectingLabel, null),
            react_1.default.createElement(DurationLabel, null),
            react_1.default.createElement(VideoContainer, null,
                react_1.default.createElement(DraggableCallContainer_1.StyledVideoElement, { ref: videoRefRemote, autoPlay: true, isVideoMuted: remoteStreamVideoIsMuted }),
                remoteStreamVideoIsMuted && (react_1.default.createElement(CenteredAvatarInConversation, null,
                    react_1.default.createElement(Avatar_1.Avatar, { size: Avatar_1.AvatarSize.XL, pubkey: ongoingCallPubkey })))),
            react_1.default.createElement(VideoContainer, null,
                react_1.default.createElement(DraggableCallContainer_1.StyledVideoElement, { ref: videoRefLocal, autoPlay: true, muted: true, isVideoMuted: localStreamVideoIsMuted }),
                localStreamVideoIsMuted && (react_1.default.createElement(CenteredAvatarInConversation, null,
                    react_1.default.createElement(Avatar_1.Avatar, { size: Avatar_1.AvatarSize.XL, pubkey: ourPubkey })))),
            react_1.default.createElement(CallButtons_1.CallWindowControls, { currentConnectedAudioInputs: currentConnectedAudioInputs, currentConnectedCameras: currentConnectedCameras, isAudioMuted: isAudioMuted, currentConnectedAudioOutputs: currentConnectedAudioOutputs, isAudioOutputMuted: isAudioOutputMuted, localStreamVideoIsMuted: localStreamVideoIsMuted, remoteStreamVideoIsMuted: remoteStreamVideoIsMuted, isFullScreen: isInFullScreen }))));
};
exports.InConversationCallContainer = InConversationCallContainer;
