"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BchatRightPanelWithDetails = void 0;
const react_1 = __importStar(require("react"));
const icon_1 = require("../icon");
const lodash_1 = __importDefault(require("lodash"));
const useInterval_1 = __importDefault(require("react-use/lib/useInterval"));
const react_redux_1 = require("react-redux");
const data_1 = require("../../data/data");
const conversationInteractions_1 = require("../../interactions/conversationInteractions");
const bchat_1 = require("../../bchat");
const conversations_1 = require("../../state/ducks/conversations");
const conversations_2 = require("../../state/selectors/conversations");
const timerOptions_1 = require("../../state/selectors/timerOptions");
const Avatar_1 = require("../avatar/Avatar");
const BchatDropdown_1 = require("../basic/BchatDropdown");
const Text_1 = require("../basic/Text");
const MediaGallery_1 = require("./media-gallery/MediaGallery");
const MessageAttachment_1 = require("../../types/MessageAttachment");
const useParamSelector_1 = require("../../hooks/useParamSelector");
async function getMediaGalleryProps(conversationId) {
    const rawMedia = await (0, data_1.getMessagesWithVisualMediaAttachments)(conversationId, bchat_1.Constants.CONVERSATION.DEFAULT_MEDIA_FETCH_COUNT);
    const rawDocuments = await (0, data_1.getMessagesWithFileAttachments)(conversationId, bchat_1.Constants.CONVERSATION.DEFAULT_DOCUMENTS_FETCH_COUNT);
    const media = lodash_1.default.flatten(rawMedia.map(attributes => {
        const { attachments, source, id, timestamp, serverTimestamp, received_at } = attributes;
        return (attachments || [])
            .filter((attachment) => attachment.thumbnail && !attachment.pending && !attachment.error)
            .map((attachment, index) => {
            const { thumbnail } = attachment;
            const mediaItem = {
                objectURL: (0, MessageAttachment_1.getAbsoluteAttachmentPath)(attachment.path),
                thumbnailObjectUrl: thumbnail ? (0, MessageAttachment_1.getAbsoluteAttachmentPath)(thumbnail.path) : undefined,
                contentType: attachment.contentType || '',
                index,
                messageTimestamp: timestamp || serverTimestamp || received_at || 0,
                messageSender: source,
                messageId: id,
                attachment,
            };
            return mediaItem;
        });
    }));
    const documents = rawDocuments.map(attributes => {
        if (!attributes.attachments?.length) {
            return null;
        }
        const attachment = attributes.attachments[0];
        const { source, id, timestamp, serverTimestamp, received_at } = attributes;
        return {
            contentType: attachment.contentType,
            index: 0,
            attachment,
            messageTimestamp: timestamp || serverTimestamp || received_at || 0,
            messageSender: source,
            messageId: id,
        };
    });
    return {
        media,
        documents: lodash_1.default.compact(documents),
    };
}
const HeaderItem = () => {
    const selectedConversation = (0, react_redux_1.useSelector)(conversations_2.getSelectedConversation);
    const dispatch = (0, react_redux_1.useDispatch)();
    if (!selectedConversation) {
        return null;
    }
    const { id, profileName } = selectedConversation;
    return (react_1.default.createElement("div", { className: "group-settings-header" },
        react_1.default.createElement("div", { className: "group-settings-header-avatarBox" },
            react_1.default.createElement(Avatar_1.Avatar, { size: Avatar_1.AvatarSize.XL, pubkey: id }),
            react_1.default.createElement("p", null, profileName)),
        react_1.default.createElement("div", { onClick: () => dispatch((0, conversations_1.closeRightPanel)()), className: "group-settings-header-closeBox" },
            react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "13.833", height: "13.822", viewBox: "0 0 17.833 17.822" },
                react_1.default.createElement("path", { id: "close", d: "M5.1,3.99A1.113,1.113,0,0,0,4.327,5.9l7.005,7.005L4.327,19.912A1.113,1.113,0,1,0,5.9,21.486l7.005-7.005,7.005,7.005a1.113,1.113,0,1,0,1.574-1.574l-7.005-7.005L21.486,5.9a1.113,1.113,0,1,0-1.574-1.574l-7.005,7.005L5.9,4.327A1.113,1.113,0,0,0,5.1,3.99Z", transform: "translate(-3.99 -3.99)" })))));
};
const BchatRightPanelWithDetails = () => {
    const [documents, setDocuments] = (0, react_1.useState)([]);
    const [media, setMedia] = (0, react_1.useState)([]);
    const selectedConversation = (0, react_redux_1.useSelector)(conversations_2.getSelectedConversation);
    const isShowing = (0, react_redux_1.useSelector)(conversations_2.isRightPanelShowing);
    (0, react_1.useEffect)(() => {
        let isRunning = true;
        if (isShowing && selectedConversation) {
            void getMediaGalleryProps(selectedConversation.id).then(results => {
                if (isRunning) {
                    if (!lodash_1.default.isEqual(documents, results.documents)) {
                        setDocuments(results.documents);
                    }
                    if (!lodash_1.default.isEqual(media, results.media)) {
                        setMedia(results.media);
                    }
                }
            });
        }
        return () => {
            isRunning = false;
            return;
        };
    }, [isShowing, selectedConversation?.id]);
    (0, useInterval_1.default)(async () => {
        if (isShowing && selectedConversation) {
            const results = await getMediaGalleryProps(selectedConversation.id);
            if (results.documents.length !== documents.length || results.media.length !== media.length) {
                setDocuments(results.documents);
                setMedia(results.media);
            }
        }
    }, 10000);
    if (!selectedConversation) {
        return null;
    }
    const { id, subscriberCount, name, isKickedFromGroup, left, isPublic, weAreAdmin, isBlocked, isGroup, isPrivate, } = selectedConversation;
    const username = String((0, useParamSelector_1.useConversationUsername)(id));
    const showMemberCount = !!(subscriberCount && subscriberCount > 0);
    const commonNoShow = isKickedFromGroup || left || isBlocked;
    const hasDisappearingMessages = !isPublic && !commonNoShow;
    const leaveGroupString = isPublic
        ? window.i18n('deleteMessages')
        : isKickedFromGroup
            ? window.i18n('youGotKickedFromGroup')
            : left
                ? window.i18n('youLeftTheGroup')
                : window.i18n('leaveGroup');
    const timerOptions = (0, react_redux_1.useSelector)(timerOptions_1.getTimerOptions).timerOptions;
    const disappearingMessagesOptions = timerOptions.map(option => {
        return {
            content: option.name,
            onClick: () => {
                void (0, conversationInteractions_1.setDisappearingMessagesByConvoId)(id, option.value);
            },
        };
    });
    const showUpdateGroupNameButton = isGroup && (!isPublic || (isPublic && weAreAdmin)) && !commonNoShow;
    const showAddRemoveModeratorsButton = weAreAdmin && !commonNoShow && isPublic;
    const showUpdateGroupMembersButton = !isPublic && isGroup && !commonNoShow;
    const deleteConvoAction = isPublic
        ? () => {
            (0, conversationInteractions_1.deleteAllMessagesByConvoIdWithConfirmation)(id);
        }
        : () => {
            (0, conversationInteractions_1.showLeaveGroupByConvoId)(id, username);
        };
    return (react_1.default.createElement("div", { className: "group-settings" },
        react_1.default.createElement(HeaderItem, null),
        isPrivate && (react_1.default.createElement("div", { className: "group-settings-header-chatIdBox" },
            react_1.default.createElement("p", null, "BChat ID:"),
            react_1.default.createElement("div", null, id))),
        react_1.default.createElement("div", { className: "group-settings-nameEditBox" },
            react_1.default.createElement("h2", { "data-testid": "right-panel-group-name" }, name),
            showUpdateGroupNameButton && (react_1.default.createElement(icon_1.BchatIconButton, { iconType: "pencil", iconSize: "medium", iconColor: "#128b17", onClick: async () => {
                    await (0, conversationInteractions_1.showUpdateGroupNameByConvoId)(id);
                }, dataTestId: "edit-profile-icon" }))),
        showMemberCount && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(Text_1.SpacerLG, null),
            react_1.default.createElement("div", { role: "button", className: "subtle", style: { textAlign: 'center' } }, window.i18n('members', [`${subscriberCount}`])),
            react_1.default.createElement(Text_1.SpacerLG, null))),
        showAddRemoveModeratorsButton && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("div", { className: "group-settings-item", role: "button", onClick: () => {
                    (0, conversationInteractions_1.showAddModeratorsByConvoId)(id);
                } }, window.i18n('addModerators')),
            react_1.default.createElement("div", { className: "group-settings-item", role: "button", onClick: () => {
                    (0, conversationInteractions_1.showRemoveModeratorsByConvoId)(id);
                } }, window.i18n('removeModerators')))),
        showUpdateGroupMembersButton && (react_1.default.createElement("div", { className: "group-settings-item", role: "button", onClick: () => {
                if (selectedConversation) {
                    (0, conversationInteractions_1.showInviteContactByConvoId)(selectedConversation.id);
                }
            } },
            react_1.default.createElement("div", { className: "invite-friends-container", style: { marginRight: '10px' }, onClick: () => {
                    if (selectedConversation) {
                        (0, conversationInteractions_1.showInviteContactByConvoId)(selectedConversation.id);
                    }
                } },
                react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "21.266", height: "21.266", viewBox: "0 0 21.266 21.266" },
                    react_1.default.createElement("path", { id: "add_member", "data-name": "add member", d: "M9.595,2a5.318,5.318,0,0,0-2.967,9.731A7.617,7.617,0,0,0,2,18.709H3.519a6.065,6.065,0,0,1,6.076-6.076,6,6,0,0,1,2.872.736,6.169,6.169,0,1,0,1.163-1.092,7.7,7.7,0,0,0-1.068-.546A5.318,5.318,0,0,0,9.595,2Zm0,1.519a3.8,3.8,0,1,1-3.8,3.8A3.786,3.786,0,0,1,9.595,3.519Zm7.595,9.114a4.557,4.557,0,1,1-4.557,4.557A4.547,4.547,0,0,1,17.19,12.633Zm-.759,1.519V16.43H14.152v1.519H16.43v2.278h1.519V17.949h2.278V16.43H17.949V14.152Z", transform: "translate(-2 -2)", fill: "#128b17" }))),
            window.i18n('addingContacts'))),
        showUpdateGroupMembersButton && (react_1.default.createElement("div", { className: "group-settings-item", role: "button", onClick: async () => {
                await (0, conversationInteractions_1.showUpdateGroupMembersByConvoId)(id);
            } },
            react_1.default.createElement("div", { className: "invite-friends-container", style: { marginRight: '10px' } },
                react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "26.154", height: "15.455", viewBox: "0 0 26.154 15.455" },
                    react_1.default.createElement("path", { id: "group_members", d: "M12.133,15a4.722,4.722,0,0,0-3.564,1.514,5.006,5.006,0,0,0,0,6.483,4.434,4.434,0,0,0,1.135.817A7.1,7.1,0,0,0,5,30.455H7.378a4.755,4.755,0,1,1,9.511,0h2.378a4.755,4.755,0,1,1,9.511,0h2.378a7.1,7.1,0,0,0-4.7-6.641A4.434,4.434,0,0,0,27.585,23a5.006,5.006,0,0,0,0-6.483,4.953,4.953,0,0,0-7.128,0,5.006,5.006,0,0,0,0,6.483,4.434,4.434,0,0,0,1.135.817,7.04,7.04,0,0,0-3.515,2.942,7.04,7.04,0,0,0-3.515-2.942A4.434,4.434,0,0,0,15.7,23a5.006,5.006,0,0,0,0-6.483A4.722,4.722,0,0,0,12.133,15Zm0,2.378a2.189,2.189,0,0,1,1.786.715,2.63,2.63,0,0,1,0,3.325,2.189,2.189,0,0,1-1.786.715,2.189,2.189,0,0,1-1.786-.715,2.63,2.63,0,0,1,0-3.325A2.189,2.189,0,0,1,12.133,17.378Zm11.888,0a2.189,2.189,0,0,1,1.786.715,2.63,2.63,0,0,1,0,3.325,2.189,2.189,0,0,1-1.786.715,2.189,2.189,0,0,1-1.786-.715,2.63,2.63,0,0,1,0-3.325A2.189,2.189,0,0,1,24.021,17.378Z", transform: "translate(-5 -15)", fill: "#128b17" }))),
            window.i18n('groupMembers'))),
        hasDisappearingMessages && (react_1.default.createElement("div", { style: { borderBottom: '2px solid #262631', width: '100%' } },
            react_1.default.createElement(BchatDropdown_1.BchatDropdown, { label: window.i18n('disappearingMessages'), options: disappearingMessagesOptions }))),
        react_1.default.createElement(MediaGallery_1.MediaGallery, { documents: documents, media: media }),
        isGroup && (react_1.default.createElement("div", { style: { marginBottom: '14px', width: '90%', borderRadius: '12px' }, onClick: deleteConvoAction },
            react_1.default.createElement("div", { className: "group-settings__leaveBtn" },
                react_1.default.createElement(icon_1.BchatIcon, { iconType: isPublic ? 'delete' : 'leaveGroup', iconSize: "tiny", iconColor: "#fc222f", iconRotation: isPublic ? 0 : 180 }),
                react_1.default.createElement("div", { style: { marginLeft: '5px' } }, leaveGroupString))))));
};
exports.BchatRightPanelWithDetails = BchatRightPanelWithDetails;
