"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutgoingMessageStatus = void 0;
const electron_1 = require("electron");
const react_1 = __importDefault(require("react"));
const styled_components_1 = __importDefault(require("styled-components"));
const icon_1 = require("../../../icon");
const MessageStatusSendingContainer = styled_components_1.default.div `
  display: inline-block;
  align-self: flex-end;
  margin-bottom: 2px;
  margin-inline-start: 5px;
  cursor: pointer;
`;
const MessageStatusSending = ({ dataTestId }) => {
    const iconColor = 'var(--color-text)';
    return (react_1.default.createElement(MessageStatusSendingContainer, { "data-testid": dataTestId, "data-testtype": "sending" },
        react_1.default.createElement(icon_1.BchatIcon, { rotateDuration: 2, iconColor: iconColor, iconType: "sending", iconSize: "tiny" })));
};
const MessageStatusSent = ({ dataTestId }) => {
    const iconColor = 'var(--color-text)';
    return (react_1.default.createElement(MessageStatusSendingContainer, { "data-testid": dataTestId, "data-testtype": "sent" },
        react_1.default.createElement(icon_1.BchatIcon, { iconColor: iconColor, iconType: "circleCheck", iconSize: "tiny" })));
};
const MessageStatusRead = ({ dataTestId }) => {
    const iconColor = 'var(--color-text)';
    return (react_1.default.createElement(MessageStatusSendingContainer, { "data-testid": dataTestId, "data-testtype": "read" },
        react_1.default.createElement(icon_1.BchatIcon, { iconColor: iconColor, iconType: "doubleCheckCircleFilled", iconSize: "tiny" })));
};
const MessageStatusError = ({ dataTestId }) => {
    const showDebugLog = () => {
        electron_1.ipcRenderer.send('show-debug-log');
    };
    return (react_1.default.createElement(MessageStatusSendingContainer, { "data-testid": dataTestId, "data-testtype": "failed", onClick: showDebugLog, title: window.i18n('sendFailed') },
        react_1.default.createElement(icon_1.BchatIcon, { iconColor: 'var(--color-destructive', iconType: "error", iconSize: "tiny" })));
};
const OutgoingMessageStatus = (props) => {
    const { status, dataTestId } = props;
    switch (status) {
        case 'sending':
            return react_1.default.createElement(MessageStatusSending, { dataTestId: dataTestId });
        case 'sent':
            return react_1.default.createElement(MessageStatusSent, { dataTestId: dataTestId });
        case 'read':
            return react_1.default.createElement(MessageStatusRead, { dataTestId: dataTestId });
        case 'error':
            return react_1.default.createElement(MessageStatusError, { dataTestId: dataTestId });
        default:
            return null;
    }
};
exports.OutgoingMessageStatus = OutgoingMessageStatus;
