"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericReadableMessage = void 0;
const classnames_1 = __importDefault(require("classnames"));
const react_1 = __importStar(require("react"));
const react_contexify_1 = require("react-contexify");
const react_redux_1 = require("react-redux");
const useInterval_1 = __importDefault(require("react-use/lib/useInterval"));
const data_1 = require("../../../../data/data");
const conversations_1 = require("../../../../bchat/conversations");
const conversations_2 = require("../../../../state/ducks/conversations");
const conversations_3 = require("../../../../state/selectors/conversations");
const timer_1 = require("../../../../util/timer");
const ExpireTimer_1 = require("../../ExpireTimer");
const MessageAvatar_1 = require("../message-content/MessageAvatar");
const MessageContentWithStatus_1 = require("../message-content/MessageContentWithStatus");
const ReadableMessage_1 = require("./ReadableMessage");
const BchatIcon_1 = require("../../../icon/BchatIcon");
const EXPIRATION_CHECK_MINIMUM = 2000;
function useIsExpired(props) {
    const { convoId, messageId, expirationLength, expirationTimestamp, isExpired: isExpiredProps, } = props;
    const dispatch = (0, react_redux_1.useDispatch)();
    const [isExpired] = (0, react_1.useState)(isExpiredProps);
    const checkExpired = (0, react_1.useCallback)(async () => {
        const now = Date.now();
        if (!expirationTimestamp || !expirationLength) {
            return;
        }
        if (isExpired || now >= expirationTimestamp) {
            await (0, data_1.removeMessage)(messageId);
            if (convoId) {
                dispatch((0, conversations_2.messageExpired)({
                    conversationKey: convoId,
                    messageId,
                }));
                const convo = (0, conversations_1.getConversationController)().get(convoId);
                convo?.updateLastMessage();
            }
        }
    }, [expirationTimestamp, expirationLength, isExpired, messageId, convoId]);
    let checkFrequency = null;
    if (expirationLength) {
        const increment = (0, timer_1.getIncrement)(expirationLength || EXPIRATION_CHECK_MINIMUM);
        checkFrequency = Math.max(EXPIRATION_CHECK_MINIMUM, increment);
    }
    (0, react_1.useEffect)(() => {
        void checkExpired();
    }, []);
    (0, useInterval_1.default)(checkExpired, checkFrequency);
    return { isExpired };
}
const GenericReadableMessage = (props) => {
    const dispatch = (0, react_redux_1.useDispatch)();
    const msgProps = (0, react_redux_1.useSelector)(state => (0, conversations_3.getGenericReadableMessageSelectorProps)(state, props.messageId));
    const isSelectionMode = (0, react_redux_1.useSelector)(conversations_3.isMessageSelectionMode);
    const expiringProps = {
        convoId: msgProps?.convoId,
        expirationLength: msgProps?.expirationLength,
        messageId: props.messageId,
        expirationTimestamp: msgProps?.expirationTimestamp,
        isExpired: msgProps?.isExpired,
    };
    const { isExpired } = useIsExpired(expiringProps);
    const isMessageSelected = (0, react_redux_1.useSelector)(state => (0, conversations_3.getIsMessageSelected)(state, props.messageId));
    const multiSelectMode = (0, react_redux_1.useSelector)(conversations_3.isMessageSelectionMode);
    const handleContextMenu = (0, react_1.useCallback)((e) => {
        const enableContextMenu = !multiSelectMode && !msgProps?.isKickedFromGroup;
        if (enableContextMenu) {
            react_contexify_1.contextMenu.hideAll();
            react_contexify_1.contextMenu.show({
                id: props.ctxMenuID,
                event: e,
            });
        }
    }, [props.ctxMenuID, multiSelectMode, msgProps?.isKickedFromGroup]);
    const { messageId, isDetailView } = props;
    if (!msgProps) {
        return null;
    }
    const { direction, conversationType, receivedAt, isUnread, expirationLength, expirationTimestamp, } = msgProps;
    if (isExpired) {
        return null;
    }
    const selected = isMessageSelected || false;
    const isGroup = conversationType === 'group';
    const isIncoming = direction === 'incoming';
    const onSelect = (0, react_1.useCallback)((messageId) => {
        dispatch((0, conversations_2.toggleSelectedMessageId)(messageId));
    }, [messageId]);
    return (react_1.default.createElement(ReadableMessage_1.ReadableMessage, { messageId: messageId, className: (0, classnames_1.default)('bchat-message-wrapper', selected && 'message-selected', isGroup && 'public-chat-message-wrapper', isIncoming ? 'bchat-message-wrapper-incoming' : 'bchat-message-wrapper-outgoing'), onContextMenu: handleContextMenu, receivedAt: receivedAt, isUnread: !!isUnread, key: `readable-message-${messageId}` },
        react_1.default.createElement("div", { className: 'message-box', onClick: () => isSelectionMode && onSelect(messageId) },
            react_1.default.createElement("div", { className: (0, classnames_1.default)(isSelectionMode && !selected && 'checkedCircle') }, selected && react_1.default.createElement("div", { className: 'isSelected' },
                react_1.default.createElement(BchatIcon_1.BchatIcon, { iconType: "check", iconColor: '#11c119', iconSize: 23 }))),
            react_1.default.createElement(MessageAvatar_1.MessageAvatar, { messageId: messageId }),
            expirationLength && expirationTimestamp && (react_1.default.createElement(ExpireTimer_1.ExpireTimer, { isCorrectSide: !isIncoming, expirationLength: expirationLength, expirationTimestamp: expirationTimestamp })),
            react_1.default.createElement(MessageContentWithStatus_1.MessageContentWithStatuses, { ctxMenuID: props.ctxMenuID, messageId: messageId, isDetailView: isDetailView, dataTestId: `message-content-${messageId}` }),
            expirationLength && expirationTimestamp && (react_1.default.createElement(ExpireTimer_1.ExpireTimer, { isCorrectSide: isIncoming, expirationLength: expirationLength, expirationTimestamp: expirationTimestamp })))));
};
exports.GenericReadableMessage = GenericReadableMessage;
