"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CallNotification = void 0;
const react_1 = __importDefault(require("react"));
const react_redux_1 = require("react-redux");
const types_1 = require("../../../../../bchat/types");
const conversations_1 = require("../../../../../state/selectors/conversations");
const ReadableMessage_1 = require("../ReadableMessage");
const NotificationBubble_1 = require("./NotificationBubble");
const style = {
    'missed-call': {
        notificationTextKey: 'callMissed',
        iconType: 'callMissed',
        iconColor: 'var(--color-destructive)',
        bgColor: 'transparent',
    },
    'started-call': {
        notificationTextKey: 'startedACall',
        iconType: 'callOutgoing',
        iconColor: '#136ef8',
        bgColor: 'transparent',
    },
    'answered-a-call': {
        notificationTextKey: 'answeredACall',
        iconType: 'callIncoming',
        iconColor: '#136ef8',
        bgColor: 'transparent',
    },
};
const CallNotification = (props) => {
    const { messageId, receivedAt, isUnread, notificationType } = props;
    const selectedConvoProps = (0, react_redux_1.useSelector)(conversations_1.getSelectedConversation);
    const displayName = selectedConvoProps?.profileName ||
        selectedConvoProps?.name ||
        (selectedConvoProps?.id && types_1.PubKey.shorten(selectedConvoProps?.id));
    const styleItem = style[notificationType];
    const notificationText = window.i18n(styleItem.notificationTextKey, [displayName || 'Unknown']);
    if (!window.i18n(styleItem.notificationTextKey)) {
        throw new Error(`invalid i18n key ${styleItem.notificationTextKey}`);
    }
    const iconType = styleItem.iconType;
    const iconColor = styleItem.iconColor;
    const bgColor = styleItem.bgColor;
    return (react_1.default.createElement(ReadableMessage_1.ReadableMessage, { messageId: messageId, receivedAt: receivedAt, isUnread: isUnread, key: `readable-message-${messageId}` },
        react_1.default.createElement(NotificationBubble_1.NotificationBubble, { notificationText: notificationText, iconType: iconType, bgColor: bgColor, iconColor: iconColor })));
};
exports.CallNotification = CallNotification;
