"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BchatPasswordDialog = void 0;
const react_1 = __importDefault(require("react"));
const util_1 = require("../../util");
const utils_1 = require("../../bchat/utils");
const data_1 = require("../../data/data");
const Text_1 = require("../basic/Text");
const auto_bind_1 = __importDefault(require("auto-bind"));
const modalDialog_1 = require("../../state/ducks/modalDialog");
const BchatButton_1 = require("../basic/BchatButton");
const BchatWrapperModal_1 = require("../BchatWrapperModal");
const passwordUtils_1 = require("../../util/passwordUtils");
class BchatPasswordDialog extends react_1.default.Component {
    passportInput = null;
    constructor(props) {
        super(props);
        this.state = {
            error: null,
            currentPasswordEntered: null,
            currentPasswordConfirmEntered: null,
            currentPasswordRetypeEntered: null,
        };
        (0, auto_bind_1.default)(this);
    }
    componentDidMount() {
        setTimeout(() => {
            this.passportInput && this.passportInput.focus();
        }, 1);
    }
    render() {
        const { passwordAction } = this.props;
        const placeholders = passwordAction === 'change'
            ? [
                window.i18n('currentPassword'),
                window.i18n('enterPassword'),
                window.i18n('confirmPassword'),
            ]
            : [window.i18n('removePasswordDisc'), window.i18n('confirmPassword')];
        const confirmButtonColor = BchatButton_1.BchatButtonColor.Green;
        const localizedKeyAction = passwordAction === 'change'
            ? 'changePassword'
            : passwordAction === 'remove'
                ? 'removePassword'
                : 'setPassword';
        return (react_1.default.createElement(BchatWrapperModal_1.BchatWrapperModal, { title: window.i18n(localizedKeyAction), onClose: this.closeDialog },
            react_1.default.createElement(Text_1.SpacerSM, null),
            react_1.default.createElement("div", { className: "bchat-modal__input-group" },
                react_1.default.createElement("input", { type: "password", id: "password-modal-input", style: { borderBottom: '2px solid var(--color-password-borderBottom)', borderRadius: 0 }, ref: input => {
                        this.passportInput = input;
                    }, placeholder: placeholders[0], onKeyUp: this.onPasswordInput, "data-testid": "password-input", maxLength: 26 }),
                passwordAction !== 'remove' && (react_1.default.createElement("input", { type: "password", id: "password-modal-input-confirm", style: { borderBottom: '2px solid var(--color-password-borderBottom)', borderRadius: 0 }, placeholder: placeholders[1], onKeyUp: this.onPasswordConfirmInput, "data-testid": "password-input-confirm", maxLength: 26 })),
                passwordAction === 'change' && (react_1.default.createElement("input", { type: "password", id: "password-modal-input-reconfirm", style: { borderBottom: '2px solid var(--color-password-borderBottom)', borderRadius: 0 }, placeholder: placeholders[2], onKeyUp: this.onPasswordRetypeInput, "data-testid": "password-input-reconfirm", maxLength: 26 }))),
            react_1.default.createElement(Text_1.SpacerSM, null),
            react_1.default.createElement("div", { className: "bchat-modal__button-group" },
                react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('cancel'), onClick: this.closeDialog }),
                react_1.default.createElement(BchatButton_1.BchatButton, { text: window.i18n('ok'), buttonColor: confirmButtonColor, onClick: this.setPassword }))));
    }
    async validatePasswordHash(password) {
        const hash = await (0, data_1.getPasswordHash)();
        if (hash && !(0, passwordUtils_1.matchesHash)(password, hash)) {
            return false;
        }
        return true;
    }
    validatePassword(firstPassword) {
        const errorFirstInput = (0, passwordUtils_1.validatePassword)(firstPassword);
        if (errorFirstInput !== null) {
            utils_1.ToastUtils.pushToastError('validatePassword', errorFirstInput);
            this.setState({
                error: errorFirstInput,
            });
            return false;
        }
        return true;
    }
    async handleActionSet(enteredPassword, enteredPasswordConfirm) {
        if (!this.validatePassword(enteredPassword)) {
            return;
        }
        if (enteredPassword !== enteredPasswordConfirm) {
            utils_1.ToastUtils.pushToastError('setPasswordInvalid', window.i18n('setPasswordInvalid'));
            this.setState({
                error: window.i18n('setPasswordInvalid'),
            });
            return;
        }
        await window.setPassword(enteredPassword, null);
        utils_1.ToastUtils.pushToastSuccess('setPasswordSuccessToast', window.i18n('setPasswordTitle'), window.i18n('setPasswordToastDescription'), 'lock');
        this.props.onOk();
        this.closeDialog();
    }
    async handleActionChange(oldPassword, newPassword, newConfirmedPassword) {
        if (!this.validatePassword(newPassword)) {
            return;
        }
        if ((oldPassword === newPassword) || (oldPassword === newConfirmedPassword)) {
            utils_1.ToastUtils.pushToastError('oldPasswordAndNewPasswordSame', window.i18n('oldPasswordAndNewPasswordSame'));
            this.setState({
                error: window.i18n('oldPasswordAndNewPasswordSame'),
            });
            return;
        }
        if (newPassword !== newConfirmedPassword) {
            utils_1.ToastUtils.pushToastError('passwordsDoNotMatch', window.i18n('passwordsDoNotMatch'));
            this.setState({
                error: window.i18n('passwordsDoNotMatch'),
            });
            return;
        }
        const isValidWithStoredInDB = Boolean(await this.validatePasswordHash(oldPassword));
        if (!isValidWithStoredInDB) {
            this.setState({
                error: window.i18n('changePasswordInvalid'),
            });
            return;
        }
        await window.setPassword(newPassword, oldPassword);
        utils_1.ToastUtils.pushToastSuccess('setPasswordSuccessToast', window.i18n('changePasswordTitle'), window.i18n('changePasswordToastDescription'), 'lock');
        this.props.onOk();
        this.closeDialog();
    }
    async handleActionRemove(oldPassword) {
        const isValidWithStoredInDB = Boolean(await this.validatePasswordHash(oldPassword));
        if (!isValidWithStoredInDB) {
            utils_1.ToastUtils.pushToastError('removePasswordInvalid', window.i18n('removePasswordInvalid'));
            this.setState({
                error: window.i18n('removePasswordInvalid'),
            });
            return;
        }
        await window.setPassword(null, oldPassword);
        utils_1.ToastUtils.pushToastSuccess('setPasswordSuccessToast', window.i18n('removePasswordTitle'), window.i18n('removePasswordToastDescription'));
        this.props.onOk();
        this.closeDialog();
    }
    async setPassword() {
        const { passwordAction } = this.props;
        const { currentPasswordEntered, currentPasswordConfirmEntered, currentPasswordRetypeEntered, } = this.state;
        const firstPasswordEntered = (currentPasswordEntered || '').trim();
        const secondPasswordEntered = (currentPasswordConfirmEntered || '').trim();
        const thirdPasswordEntered = (currentPasswordRetypeEntered || '').trim();
        switch (passwordAction) {
            case 'set': {
                await this.handleActionSet(firstPasswordEntered, secondPasswordEntered);
                return;
            }
            case 'change': {
                await this.handleActionChange(firstPasswordEntered, secondPasswordEntered, thirdPasswordEntered);
                return;
            }
            case 'remove': {
                await this.handleActionRemove(firstPasswordEntered);
                return;
            }
            default:
                throw (0, util_1.missingCaseError)(passwordAction);
        }
    }
    closeDialog() {
        window.inboxStore?.dispatch((0, modalDialog_1.bchatPassword)(null));
    }
    async onPasswordInput(event) {
        if (event.key === 'Enter') {
            return this.setPassword();
        }
        const currentPasswordEntered = event.target.value;
        this.setState({ currentPasswordEntered });
    }
    async onPasswordConfirmInput(event) {
        if (event.key === 'Enter') {
            return this.setPassword();
        }
        const currentPasswordConfirmEntered = event.target.value;
        this.setState({ currentPasswordConfirmEntered });
    }
    async onPasswordRetypeInput(event) {
        if (event.key === 'Enter') {
            return this.setPassword();
        }
        const currentPasswordRetypeEntered = event.target.value;
        this.setState({ currentPasswordRetypeEntered });
    }
}
exports.BchatPasswordDialog = BchatPasswordDialog;
