"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyBchatID = exports.EditProfileDialog = void 0;
const react_1 = __importDefault(require("react"));
const react_qr_svg_1 = require("react-qr-svg");
const Avatar_1 = require("../avatar/Avatar");
const PillDivider_1 = require("../basic/PillDivider");
const utils_1 = require("../../bchat/utils");
const conversation_1 = require("../../models/conversation");
const conversations_1 = require("../../bchat/conversations");
const auto_bind_1 = __importDefault(require("auto-bind"));
const modalDialog_1 = require("../../state/ducks/modalDialog");
const conversationInteractions_1 = require("../../interactions/conversationInteractions");
const icon_1 = require("../icon");
const RegistrationStages_1 = require("../registration/RegistrationStages");
const BchatWrapperModal_1 = require("../BchatWrapperModal");
const VisualAttachment_1 = require("../../types/attachments/VisualAttachment");
const String_1 = require("../../bchat/utils/String");
const storage_1 = require("../../util/storage");
const styled_components_1 = __importDefault(require("styled-components"));
const ActionsPanel_1 = require("../leftpane/ActionsPanel");
const QRView = ({ bchatID }) => {
    return (react_1.default.createElement("div", { className: "qr-image" },
        react_1.default.createElement(react_qr_svg_1.QRCode, { value: bchatID, bgColor: "#FFFFFF", fgColor: "#1B1B1B", level: "L" })));
};
class EditProfileDialog extends react_1.default.Component {
    convo;
    constructor(props) {
        super(props);
        (0, auto_bind_1.default)(this);
        this.convo = (0, conversations_1.getConversationController)().get(utils_1.UserUtils.getOurPubKeyStrFromCache());
        this.state = {
            profileName: this.convo.getProfileName() || '',
            setProfileName: this.convo.getProfileName() || '',
            oldAvatarPath: this.convo.getAvatarPath() || '',
            newAvatarObjectUrl: null,
            mode: 'default',
            loading: false,
        };
    }
    componentDidMount() {
        window.addEventListener('keyup', this.onKeyUp);
    }
    componentWillUnmount() {
        window.removeEventListener('keyup', this.onKeyUp);
    }
    render() {
        const i18n = window.i18n;
        const viewDefault = this.state.mode === 'default';
        const viewEdit = this.state.mode === 'edit';
        const viewQR = this.state.mode === 'qr';
        const bchatID = utils_1.UserUtils.getOurPubKeyStrFromCache();
        const backButton = viewEdit || viewQR
            ? [
                {
                    iconType: 'chevron',
                    iconRotation: 90,
                    onClick: () => {
                        this.setState({ mode: 'default' });
                    },
                },
            ]
            : undefined;
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("div", { className: "edit-profile-dialog", "data-testid": "edit-profile-dialog" },
                react_1.default.createElement(BchatWrapperModal_1.BchatWrapperModal, { title: i18n('editProfileModalTitle'), onClose: this.closeDialog, showHeader: false, headerIconButtons: backButton, showExitIcon: true },
                    this.state.loading && (react_1.default.createElement(Loader, null,
                        react_1.default.createElement("div", { className: "edit-profile-dialog-modalLoader" },
                            react_1.default.createElement("img", { src: 'images/bchat/Load_animation.gif', style: { width: '150px', height: '150px' } })))),
                    react_1.default.createElement("div", { className: 'profileClose' },
                        react_1.default.createElement(icon_1.BchatIconButton, { iconType: "exit", iconSize: "tiny", onClick: this.closeDialog, dataTestId: "modal-close-button" })),
                    (viewDefault || viewQR) && this.renderDefaultView(),
                    viewEdit && this.renderEditView(),
                    react_1.default.createElement("div", { className: "bchat-id-section" },
                        react_1.default.createElement(PillDivider_1.PillDivider, null),
                        !viewQR ? this.renderAddressView({ bchatID }) : this.renderQrView({ bchatID }))))));
    }
    renderProfileHeader() {
        return (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("div", { className: "avatar-center" },
                react_1.default.createElement("div", { className: "avatar-center-inner" },
                    this.renderAvatar(),
                    react_1.default.createElement("div", { className: "image-upload-section", role: "button", "data-testid": "image-upload-section" }),
                    react_1.default.createElement("div", { "data-tip": "Edit", "data-place": "right", "data-offset": "{'top':15,'left':10}", style: {
                            background: `url(images/bchat/camera.svg) no-repeat`,
                            width: '30px',
                            height: '30px',
                            position: 'relative',
                            justifyContent: 'center',
                            backgroundSize: '32px',
                            top: '56px',
                            right: '20px',
                            alignItems: 'center',
                            cursor: 'pointer',
                        }, onClick: this.fireInputEvent, role: "button", "data-testid": "image-upload-section" },
                        react_1.default.createElement(ActionsPanel_1.BchatToolTip, { place: "top", effect: "solid" }))))));
    }
    async fireInputEvent() {
        const scaledAvatarUrl = await (0, VisualAttachment_1.pickFileForAvatar)();
        if (scaledAvatarUrl) {
            this.setState({
                newAvatarObjectUrl: scaledAvatarUrl,
                mode: 'edit',
                loading: true,
            });
            this.onClickOK();
        }
    }
    renderDefaultView() {
        const name = this.state.setProfileName || this.state.profileName;
        const mode = this.state.mode == 'qr' ? 'default' : 'qr';
        const SwicthContact = mode == 'qr' ? 'qr_code' : 'profile_share';
        return (react_1.default.createElement(react_1.default.Fragment, null,
            this.renderProfileHeader(),
            react_1.default.createElement("div", { className: "profile-name-uneditable" },
                react_1.default.createElement("div", { style: {
                        display: 'flex',
                        width: '86%',
                        justifyContent: 'center',
                    } },
                    react_1.default.createElement("p", { "data-testid": "your-profile-name" }, name),
                    react_1.default.createElement(icon_1.BchatIconButton, { iconType: "pencil", iconSize: "medium", iconColor: "#128b17", onClick: () => {
                            this.setState({ mode: 'edit' });
                        }, dataTestId: "edit-profile-icon" })),
                react_1.default.createElement("div", { className: "qr-icon-btn", style: {
                        cursor: 'pointer',
                    }, onClick: () => {
                        this.setState(state => ({ ...state, mode: mode }));
                    }, role: "button" }, SwicthContact === 'qr_code' ? (react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "29.313", height: "29.313", viewBox: "0 0 29.313 29.313" },
                    react_1.default.createElement("path", { id: "icons8-qr_code", d: "M4.141,1A3.152,3.152,0,0,0,1,4.141V8.328a3.152,3.152,0,0,0,3.141,3.141H5.187v2.094H7.281V11.469H8.328a3.152,3.152,0,0,0,3.141-3.141V4.141A3.152,3.152,0,0,0,8.328,1ZM7.281,13.563v2.094H9.375V13.563Zm0,2.094H5.187V17.75H7.281Zm-2.094,0V13.563H3.094v2.094ZM22.984,1a3.152,3.152,0,0,0-3.141,3.141V8.328a3.152,3.152,0,0,0,3.141,3.141h1.047v2.094h2.094V11.469h1.047a3.152,3.152,0,0,0,3.141-3.141V4.141A3.152,3.152,0,0,0,27.172,1Zm3.141,12.563v2.094h2.094V13.563Zm0,2.094H24.031V17.75h2.094Zm0,2.094v2.094h2.094V17.75Zm0,2.094H24.031v2.094h2.094Zm-2.094,0V17.75H21.938v2.094ZM4.141,3.094H8.328A1.033,1.033,0,0,1,9.375,4.141V8.328A1.033,1.033,0,0,1,8.328,9.375H4.141A1.033,1.033,0,0,1,3.094,8.328V4.141A1.033,1.033,0,0,1,4.141,3.094Zm11.516,0V5.187H13.563V7.281h2.094V9.375H17.75V3.094Zm7.328,0h4.188a1.033,1.033,0,0,1,1.047,1.047V8.328a1.033,1.033,0,0,1-1.047,1.047H22.984a1.033,1.033,0,0,1-1.047-1.047V4.141A1.033,1.033,0,0,1,22.984,3.094ZM5.187,5.187V7.281H7.281V5.187Zm18.844,0V7.281h2.094V5.187ZM11.469,11.469v2.094H17.75v2.094h2.094V11.469Zm2.094,4.187V17.75H11.469v2.094h8.375V17.75H15.656V15.656ZM4.141,19.844A3.152,3.152,0,0,0,1,22.984v4.188a3.152,3.152,0,0,0,3.141,3.141H8.328a3.152,3.152,0,0,0,3.141-3.141V22.984a3.152,3.152,0,0,0-3.141-3.141Zm0,2.094H8.328a1.033,1.033,0,0,1,1.047,1.047v4.188a1.033,1.033,0,0,1-1.047,1.047H4.141a1.033,1.033,0,0,1-1.047-1.047V22.984A1.033,1.033,0,0,1,4.141,21.938Zm9.422,0v6.281h2.094V21.938ZM5.187,24.031v2.094H7.281V24.031Zm14.656,0v2.094h2.094V24.031Zm2.094,2.094v2.094h2.094V26.125Zm2.094,0h2.094V24.031H24.031Zm2.094,0v2.094h2.094V26.125Z", transform: "translate(-1 -1)" }))) : (react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "29", height: "30", viewBox: "0 0 22.646 22.646" },
                    react_1.default.createElement("path", { id: "icons8-copybook", d: "M10.065,5A3.291,3.291,0,0,0,6.788,8.278v.894H5.894a.894.894,0,1,0,0,1.788H9.469a.894.894,0,1,0,0-1.788H8.576V8.278a1.477,1.477,0,0,1,1.49-1.49H23.176a1.477,1.477,0,0,1,1.49,1.49v.449a.894.894,0,0,0,0,.29V12.3a.894.894,0,0,0,0,.29v2.094a.894.894,0,0,0,0,.29v3.286a.894.894,0,0,0,0,.29v2.094a.894.894,0,0,0,0,.29v3.286a.894.894,0,0,0-.008.229,1.472,1.472,0,0,1-1.482,1.407H10.065a1.477,1.477,0,0,1-1.49-1.49v-.894h.894a.894.894,0,1,0,0-1.788H8.576V20.494H6.788v1.192H5.894a.894.894,0,1,0,0,1.788h.894v.894a3.291,3.291,0,0,0,3.278,3.278H23.176a3.293,3.293,0,0,0,3.154-2.4,1.5,1.5,0,0,0,1.315-1.472V21.388a1.508,1.508,0,0,0-1.192-1.46v-.656a1.508,1.508,0,0,0,1.192-1.46V15.429a1.508,1.508,0,0,0-1.192-1.46v-.656a1.508,1.508,0,0,0,1.192-1.46V9.47A1.506,1.506,0,0,0,26.44,8.009,3.291,3.291,0,0,0,23.176,5Zm2.98,4.172a.894.894,0,0,0-.894.894v4.172a.894.894,0,0,0,.894.894h7.747a.894.894,0,0,0,.894-.894V10.065a.894.894,0,0,0-.894-.894Zm.894,1.788H19.9v2.384H13.939ZM6.788,12.151v1.192H5.894a.894.894,0,1,0,0,1.788H9.469a.894.894,0,1,0,0-1.788H8.576V12.151Zm0,4.172v1.192H5.894a.894.894,0,1,0,0,1.788H9.469a.894.894,0,1,0,0-1.788H8.576V16.323Z", transform: "translate(-5 -5)" })))))));
    }
    renderEditView() {
        const placeholderText = window.i18n('displayName');
        return (react_1.default.createElement(react_1.default.Fragment, null,
            this.renderProfileHeader(),
            react_1.default.createElement("div", { className: "profile-name" },
                react_1.default.createElement("div", { className: "inputBox" },
                    react_1.default.createElement("input", { type: "text", className: "profile-name-input", value: this.state.profileName, placeholder: placeholderText, onChange: this.onNameEdited, maxLength: RegistrationStages_1.MAX_USERNAME_LENGTH, tabIndex: 0, required: true, "aria-required": true, "data-testid": "profile-name-input" })),
                react_1.default.createElement("div", { className: "saveIcon", onClick: () => this.onClickOK() },
                    react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 24 24" },
                        react_1.default.createElement("path", { d: "M12,2C6.477,2,2,6.477,2,12c0,5.523,4.477,10,10,10s10-4.477,10-10C22,6.477,17.523,2,12,2z M17.707,9.707l-7,7 C10.512,16.902,10.256,17,10,17s-0.512-0.098-0.707-0.293l-3-3c-0.391-0.391-0.391-1.023,0-1.414s1.023-0.391,1.414,0L10,14.586 l6.293-6.293c0.391-0.391,1.023-0.391,1.414,0S18.098,9.316,17.707,9.707z" }))))));
    }
    renderAddressView(props) {
        let walletAddress = localStorage.getItem('userAddress');
        return (react_1.default.createElement("div", null,
            react_1.default.createElement("p", { className: "profile-header" }, window.i18n('BchatID')),
            react_1.default.createElement("div", { className: "bchat-id-section-display" },
                react_1.default.createElement("div", { className: "profile-value" }, props.bchatID),
                react_1.default.createElement("div", { onClick: () => copyBchatID(props.bchatID), className: "bchat-id-section-display-icon", "data-tip": "Copy", "data-place": "right", "data-offset": "{'top':17}" },
                    react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "15", height: "15", viewBox: "0 0 18.151 18.151" },
                        react_1.default.createElement("path", { id: "copy_icon", d: "M3.815,2A1.815,1.815,0,0,0,2,3.815V16.521H3.815V3.815H16.521V2Zm3.63,3.63A1.815,1.815,0,0,0,5.63,7.445V18.336a1.815,1.815,0,0,0,1.815,1.815H18.336a1.815,1.815,0,0,0,1.815-1.815V7.445A1.815,1.815,0,0,0,18.336,5.63Zm0,1.815H18.336V18.336H7.445Z", transform: "translate(-2 -2)" })))),
            react_1.default.createElement("p", { className: "profile-header" }, window.i18n('profileBeldexAddres')),
            react_1.default.createElement("div", { className: "bchat-id-section-display", style: { marginBottom: '37px' } },
                react_1.default.createElement("div", { className: "profile-value", style: { color: '#1782FF' } }, walletAddress),
                react_1.default.createElement("div", { onClick: () => copyBchatID(walletAddress), "data-tip": "Copy", "data-place": "right", "data-offset": "{'top':17}", className: "bchat-id-section-display-icon" },
                    react_1.default.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "15", height: "15", viewBox: "0 0 18.151 18.151" },
                        react_1.default.createElement("path", { id: "copy_icon", d: "M3.815,2A1.815,1.815,0,0,0,2,3.815V16.521H3.815V3.815H16.521V2Zm3.63,3.63A1.815,1.815,0,0,0,5.63,7.445V18.336a1.815,1.815,0,0,0,1.815,1.815H18.336a1.815,1.815,0,0,0,1.815-1.815V7.445A1.815,1.815,0,0,0,18.336,5.63Zm0,1.815H18.336V18.336H7.445Z", transform: "translate(-2 -2)" })))),
            react_1.default.createElement(ActionsPanel_1.BchatToolTip, { effect: "solid" })));
    }
    renderQrView(props) {
        return (react_1.default.createElement("div", { className: "qr-box-view" },
            react_1.default.createElement("img", { src: "images/bchat/Bchat_logo_QR.svg", className: "qr-center-icon" }),
            react_1.default.createElement(QRView, { bchatID: props.bchatID }),
            react_1.default.createElement("p", null, window.i18n('scanQr'))));
    }
    renderAvatar() {
        const { oldAvatarPath, newAvatarObjectUrl, profileName } = this.state;
        const userName = profileName || this.convo.id;
        return (react_1.default.createElement(Avatar_1.Avatar, { forcedAvatarPath: newAvatarObjectUrl || oldAvatarPath, forcedName: userName, size: Avatar_1.AvatarSize.XL, pubkey: this.convo.id }));
    }
    onNameEdited(event) {
        const newName = (0, String_1.sanitizeBchatUsername)(event.target.value);
        this.setState({
            profileName: newName,
        });
    }
    onKeyUp(event) {
        switch (event.key) {
            case 'Enter':
                if (this.state.mode === 'edit') {
                    this.onClickOK();
                }
                break;
            case 'Esc':
            case 'Escape':
                this.closeDialog();
                break;
            default:
        }
    }
    onClickOK() {
        const { newAvatarObjectUrl, profileName } = this.state;
        const newName = profileName ? profileName.trim() : '';
        if (newName.length === 0 || newName.length > RegistrationStages_1.MAX_USERNAME_LENGTH) {
            return;
        }
        this.setState({
            loading: true,
        }, async () => {
            await commitProfileEdits(newName, newAvatarObjectUrl);
            this.setState({
                loading: false,
                mode: 'default',
                setProfileName: this.state.profileName,
            });
        });
    }
    closeDialog() {
        window.removeEventListener('keyup', this.onKeyUp);
        window.inboxStore?.dispatch((0, modalDialog_1.editProfileModal)(null));
    }
}
exports.EditProfileDialog = EditProfileDialog;
async function commitProfileEdits(newName, scaledAvatarUrl) {
    const ourNumber = utils_1.UserUtils.getOurPubKeyStrFromCache();
    const conversation = await (0, conversations_1.getConversationController)().getOrCreateAndWait(ourNumber, conversation_1.ConversationTypeEnum.PRIVATE);
    if (scaledAvatarUrl?.length) {
        try {
            const blobContent = await (await fetch(scaledAvatarUrl)).blob();
            if (!blobContent || !blobContent.size) {
                throw new Error('Failed to fetch blob content from scaled avatar');
            }
            await (0, conversationInteractions_1.uploadOurAvatar)(await blobContent.arrayBuffer());
        }
        catch (error) {
            if (error.message && error.message.length) {
                utils_1.ToastUtils.pushToastError('edit-profile', error.message);
            }
            window.log.error('showEditProfileDialog Error ensuring that image is properly sized:', error && error.stack ? error.stack : error);
        }
        return;
    }
    await conversation.setBchatProfile({
        displayName: newName,
    });
    await conversation.commit();
    await (0, storage_1.setLastProfileUpdateTimestamp)(Date.now());
    await utils_1.SyncUtils.forceSyncConfigurationNowIfNeeded(true);
}
const Loader = styled_components_1.default.div `
  position: absolute;
  top: 0;
  display: flex;
  justify-content: center;
  /* width: 100%; */
  width: 100vw;
  height: 100%;
  align-items: center;
  z-index: 101;
`;
function copyBchatID(bchatID) {
    window.clipboard.writeText(bchatID);
    utils_1.ToastUtils.pushCopiedToClipBoard();
}
exports.copyBchatID = copyBchatID;
