"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddModeratorsDialog = void 0;
const react_1 = __importStar(require("react"));
const types_1 = require("../../bchat/types");
const utils_1 = require("../../bchat/utils");
const Flex_1 = require("../basic/Flex");
const opengroupV2_1 = require("../../bchat/apis/open_group_api/opengroupV2");
const conversations_1 = require("../../bchat/conversations");
const react_redux_1 = require("react-redux");
const modalDialog_1 = require("../../state/ducks/modalDialog");
const BchatButton_1 = require("../basic/BchatButton");
const BchatSpinner_1 = require("../basic/BchatSpinner");
const BchatWrapperModal_1 = require("../BchatWrapperModal");
const AddModeratorsDialog = (props) => {
    const { conversationId } = props;
    const dispatch = (0, react_redux_1.useDispatch)();
    const convo = (0, conversations_1.getConversationController)().get(conversationId);
    const [inputBoxValue, setInputBoxValue] = (0, react_1.useState)('');
    const [addingInProgress, setAddingInProgress] = (0, react_1.useState)(false);
    const addAsModerator = async () => {
        const pubkey = types_1.PubKey.from(inputBoxValue);
        if (!pubkey) {
            window.log.info('invalid pubkey for adding as moderator:', inputBoxValue);
            utils_1.ToastUtils.pushInvalidPubKey();
            return;
        }
        window?.log?.info(`asked to add moderator: ${pubkey.key}`);
        try {
            setAddingInProgress(true);
            let isAdded;
            const roomInfos = convo.toOpenGroupV2();
            isAdded = await opengroupV2_1.ApiV2.addModerator(pubkey, roomInfos);
            if (!isAdded) {
                window?.log?.warn('failed to add moderators:', isAdded);
                utils_1.ToastUtils.pushFailedToAddAsModerator();
            }
            else {
                window?.log?.info(`${pubkey.key} added as moderator...`);
                utils_1.ToastUtils.pushUserAddedToModerators();
                setInputBoxValue('');
            }
        }
        catch (e) {
            window?.log?.error('Got error while adding moderator:', e);
        }
        finally {
            setAddingInProgress(false);
        }
    };
    const { i18n } = window;
    const chatName = convo.get('name');
    const title = `${i18n('addModerators')}: ${chatName}`;
    const onPubkeyBoxChanges = (e) => {
        const val = e.target.value;
        setInputBoxValue(val);
    };
    return (react_1.default.createElement(BchatWrapperModal_1.BchatWrapperModal, { showExitIcon: true, title: title, onClose: () => {
            dispatch((0, modalDialog_1.updateAddModeratorsModal)(null));
        } },
        react_1.default.createElement(Flex_1.Flex, { container: true, flexDirection: "column", alignItems: "center" },
            react_1.default.createElement("p", null, "Add Moderator:"),
            react_1.default.createElement("input", { type: "text", className: "module-main-header__search__input", placeholder: i18n('enterBchatID'), dir: "auto", onChange: onPubkeyBoxChanges, disabled: addingInProgress, value: inputBoxValue }),
            react_1.default.createElement(BchatButton_1.BchatButton, { buttonType: BchatButton_1.BchatButtonType.Brand, buttonColor: BchatButton_1.BchatButtonColor.Primary, onClick: addAsModerator, text: i18n('add'), disabled: addingInProgress }),
            react_1.default.createElement(BchatSpinner_1.BchatSpinner, { loading: addingInProgress }))));
};
exports.AddModeratorsDialog = AddModeratorsDialog;
