"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LeftPane = void 0;
const react_1 = __importDefault(require("react"));
const react_redux_1 = require("react-redux");
const section_1 = require("../../state/ducks/section");
const BchatTheme_1 = require("../../state/ducks/BchatTheme");
const conversations_1 = require("../../state/selectors/conversations");
const search_1 = require("../../state/selectors/search");
const section_2 = require("../../state/selectors/section");
const userConfig_1 = require("../../state/selectors/userConfig");
const ActionsPanel_1 = require("./ActionsPanel");
const LeftPaneMessageSection_1 = require("./LeftPaneMessageSection");
const LeftPaneSettingSection_1 = require("./LeftPaneSettingSection");
const OverlayOpenGroup_1 = require("./overlay/OverlayOpenGroup");
const OverlayClosedGroup_1 = require("./overlay/OverlayClosedGroup");
const conversations_2 = require("../../state/selectors/conversations");
const BchatWalletAddressBook_1 = require("../wallet/BchatWalletAddressBook");
const InnerLeftPaneMessageSection = () => {
    const showSearch = (0, react_redux_1.useSelector)(search_1.isSearching);
    const searchResults = showSearch ? (0, react_redux_1.useSelector)(search_1.getSearchResults) : undefined;
    const lists = showSearch ? undefined : (0, react_redux_1.useSelector)(conversations_1.getLeftPaneLists);
    const messageRequestsEnabled = (0, react_redux_1.useSelector)(userConfig_1.getHideMessageRequestBanner);
    const overlayMode = (0, react_redux_1.useSelector)(section_2.getOverlayMode);
    const directContact = (0, react_redux_1.useSelector)(conversations_2.getDirectContacts);
    return (react_1.default.createElement(LeftPaneMessageSection_1.LeftPaneMessageSection, { conversations: lists?.conversations || [], contacts: lists?.contacts || [], searchResults: searchResults, messageRequestsEnabled: messageRequestsEnabled, overlayMode: overlayMode, directContact: directContact }));
};
const LeftPaneSection = () => {
    const focusedSection = (0, react_redux_1.useSelector)(section_2.getFocusedSection);
    if (focusedSection === section_1.SectionType.Message) {
        return react_1.default.createElement(InnerLeftPaneMessageSection, null);
    }
    if (focusedSection === section_1.SectionType.Closedgroup) {
        return react_1.default.createElement(OverlayClosedGroup_1.OverlayClosedGroup, null);
    }
    if (focusedSection === section_1.SectionType.Opengroup) {
        return react_1.default.createElement(OverlayOpenGroup_1.OverlayOpenGroup, null);
    }
    if (focusedSection === section_1.SectionType.Wallet) {
        return react_1.default.createElement(BchatWalletAddressBook_1.AddressBook, { from: window.i18n('contact') });
    }
    if (focusedSection === section_1.SectionType.Settings) {
        return react_1.default.createElement(LeftPaneSettingSection_1.LeftPaneSettingSection, null);
    }
    return null;
};
const AddContactFloatingIcon = () => {
    const focusedSection = (0, react_redux_1.useSelector)(section_2.getFocusedSection);
    const overlayMode = (0, react_redux_1.useSelector)(section_2.getOverlayMode);
    if (focusedSection === section_1.SectionType.Message && overlayMode !== 'message') {
        return react_1.default.createElement("div", { className: 'addContactFloating' },
            react_1.default.createElement("div", { className: 'addContactFloating-content', "data-tip": "Add Contacts", "data-offset": "{'top':80,'right':80}", "data-place": "bottom", onClick: () => window.inboxStore?.dispatch((0, section_1.setOverlayMode)('message')) },
                react_1.default.createElement("img", { src: "images/wallet/addNewChat.svg", style: { width: '23px', height: '23px' } })));
    }
    return react_1.default.createElement(react_1.default.Fragment, null);
};
const LeftPane = () => {
    return (react_1.default.createElement(BchatTheme_1.BchatTheme, null,
        react_1.default.createElement("div", { className: "module-left-pane-bchat" },
            react_1.default.createElement("div", { className: "module-left-pane" },
                react_1.default.createElement(LeftPaneSection, null),
                react_1.default.createElement(AddContactFloatingIcon, null),
                react_1.default.createElement(ActionsPanel_1.ActionsPanel, null)))));
};
exports.LeftPane = LeftPane;
