"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LeftPaneMessageSection = void 0;
const react_1 = __importDefault(require("react"));
const react_virtualized_1 = require("react-virtualized");
const ConversationListItem_1 = require("./conversation-list-item/ConversationListItem");
const SearchResults_1 = require("../search/SearchResults");
const LeftPaneSectionHeader_1 = require("./LeftPaneSectionHeader");
const auto_bind_1 = __importDefault(require("auto-bind"));
const MessageRequestsBanner_1 = require("./MessageRequestsBanner");
const BchatSearchInput_1 = require("../BchatSearchInput");
const OverlayOpenGroup_1 = require("./overlay/OverlayOpenGroup");
const OverlayMessageRequest_1 = require("./overlay/OverlayMessageRequest");
const OverlayMessage_1 = require("./overlay/OverlayMessage");
const OverlayClosedGroup_1 = require("./overlay/OverlayClosedGroup");
const section_1 = require("../../state/ducks/section");
class LeftPaneMessageSection extends react_1.default.Component {
    constructor(props) {
        super(props);
        (0, auto_bind_1.default)(this);
    }
    renderRow = ({ index, key, style }) => {
        const { conversations } = this.props;
        if (!conversations) {
            throw new Error('renderRow: Tried to render without conversations');
        }
        const conversation = conversations[index];
        if (!conversation) {
            throw new Error('renderRow: conversations selector returned element containing falsy value.');
        }
        return react_1.default.createElement(ConversationListItem_1.MemoConversationListItemWithDetails, { key: key, style: style, ...conversation });
    };
    renderList() {
        const { conversations, searchResults } = this.props;
        if (searchResults) {
            return react_1.default.createElement(SearchResults_1.SearchResults, { ...searchResults });
        }
        if (!conversations) {
            throw new Error('render: must provided conversations if no search results are provided');
        }
        const length = conversations.length;
        const listKey = 0;
        const list = (react_1.default.createElement("div", { className: "module-left-pane__list", key: listKey },
            react_1.default.createElement(react_virtualized_1.AutoSizer, null, ({ height, width }) => (react_1.default.createElement(react_virtualized_1.List, { className: "module-left-pane__virtual-list", conversations: conversations, height: height, rowCount: length, rowHeight: 80, rowRenderer: this.renderRow, width: width, autoHeight: false })))));
        return [list];
    }
    render() {
        const { overlayMode } = this.props;
        return (react_1.default.createElement("div", { className: "bchat-left-pane-section-content" },
            react_1.default.createElement(LeftPaneSectionHeader_1.LeftPaneSectionHeader, null),
            overlayMode ? this.renderClosableOverlay() : null,
            overlayMode ? null : react_1.default.createElement(react_1.default.Fragment, null, this.renderConversations())));
    }
    renderConversations() {
        const { conversations, directContact } = this.props;
        return (react_1.default.createElement("div", { className: "module-conversations-list-content" },
            directContact.length !== 0 &&
                react_1.default.createElement(BchatSearchInput_1.BchatSearchInput, null),
            react_1.default.createElement(MessageRequestsBanner_1.MessageRequestsBanner, { handleOnClick: () => {
                    window.inboxStore?.dispatch((0, section_1.setOverlayMode)('message-requests'));
                } }),
            directContact.length === 0 && conversations?.length === 0 ?
                react_1.default.createElement("div", { className: 'bchatEmptyScrBox' },
                    react_1.default.createElement("div", { className: 'addContactImg' }),
                    react_1.default.createElement("h4", { className: 'module-left-pane__empty_contact' }, window.i18n('noContactsYet')),
                    react_1.default.createElement("div", { style: { display: "flex" } },
                        react_1.default.createElement("button", { className: 'nextButton', onClick: () => window.inboxStore?.dispatch((0, section_1.setOverlayMode)('message')) }, "Add Contacts + ")))
                : this.renderList(),
            this.renderBottomButtons()));
    }
    renderClosableOverlay() {
        const { overlayMode } = this.props;
        switch (overlayMode) {
            case 'open-group':
                return react_1.default.createElement(OverlayOpenGroup_1.OverlayOpenGroup, null);
            case 'closed-group':
                return react_1.default.createElement(OverlayClosedGroup_1.OverlayClosedGroup, null);
            case 'message':
                return react_1.default.createElement(OverlayMessage_1.OverlayMessage, null);
            case 'message-requests':
                return react_1.default.createElement(OverlayMessageRequest_1.OverlayMessageRequest, { leftPane: true });
            default:
                return null;
        }
    }
    renderBottomButtons() {
        return (react_1.default.createElement("div", { className: "left-pane-contact-bottom-buttons" }));
    }
}
exports.LeftPaneMessageSection = LeftPaneMessageSection;
